@@ -51,6 +51,7 @@ from .nifti1 import Nifti1Header, Nifti1Image, Nifti1Pair
 from .nifti2 import Nifti2Header, Nifti2Image, Nifti2Pair
 from .minc1 import Minc1Image
 from .minc2 import Minc2Image
+from .cifti2 import Cifti2Header, Cifti2Image
 # Deprecated backwards compatiblity for MINC1
 from .deprecated import ModuleProxy as _ModuleProxy
 minc = _ModuleProxy('nibabel.minc')
@@ -66,6 +67,7 @@ from . import trackvis
 from . import mriutils
 from . import streamlines
 from . import viewers
+from .testing import setup_test
 
 # Note test requirement for "mock".  Requirement for "nose" tested by numpy.
 try:
