@@ -1,8 +1,12 @@
-""" Module to help with deprecating classes and modules
+""" Module to help with deprecating objects and classes
 """
 
 import warnings
 
+from .deprecator import Deprecator
+from .info import cmp_pkg_version
+
+
 class ModuleProxy(object):
     """ Proxy for module that may not yet have been imported
 
@@ -19,9 +23,11 @@ class ModuleProxy(object):
         minc = ModuleProxy('nibabel.minc')
         minc_image = minc.Minc1Image(arr, np.eye(4))
 
-    So, the ``minc`` object is a proxy that will import the required module when
-    you do attribute access and return the attributes of the imported module.
+    So, the ``minc`` object is a proxy that will import the required module
+    when you do attribute access and return the attributes of the imported
+    module.
     """
+
     def __init__(self, module_name):
         self._module_name = module_name
 
@@ -54,8 +60,24 @@ class FutureWarningMixin(object):
     FutureWarning("Please, don't use this class",)
     """
     warn_message = 'This class will be removed in future versions'
+
     def __init__(self, *args, **kwargs):
         warnings.warn(self.warn_message,
                       FutureWarning,
                       stacklevel=2)
         super(FutureWarningMixin, self).__init__(*args, **kwargs)
+
+
+class VisibleDeprecationWarning(UserWarning):
+    """ Deprecation warning that will be shown by default
+
+    Python >= 2.7 does not show standard DeprecationWarnings by default:
+
+    http://docs.python.org/dev/whatsnew/2.7.html#the-future-for-python-2-x
+
+    Use this class for cases where we do want to show deprecations by default.
+    """
+    pass
+
+
+deprecate_with_version = Deprecator(cmp_pkg_version)
