@@ -6,7 +6,7 @@ import operator
 from numbers import Integral
 from mmap import mmap
 
-from .externals.six.moves import reduce
+from six.moves import reduce
 
 import numpy as np
 
@@ -17,6 +17,22 @@ import numpy as np
 SKIP_THRESH = 2 ** 8
 
 
+class _NullLock(object):
+    """Can be used as no-function dummy object in place of ``threading.lock``.
+
+    The ``_NullLock`` is an object which can be used in place of a
+    ``threading.Lock`` object, but doesn't actually do anything.
+
+    It is used by the ``read_segments`` function in the event that a
+    ``Lock`` is not provided by the caller.
+    """
+    def __enter__(self):
+        pass
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return False
+
+
 def is_fancy(sliceobj):
     """ Returns True if sliceobj is attempting fancy indexing
 
@@ -622,7 +638,7 @@ def slicers2segments(read_slicers, in_shape, offset, itemsize):
     return all_segments
 
 
-def read_segments(fileobj, segments, n_bytes):
+def read_segments(fileobj, segments, n_bytes, lock=None):
     """ Read `n_bytes` byte data implied by `segments` from `fileobj`
 
     Parameters
@@ -634,6 +650,14 @@ def read_segments(fileobj, segments, n_bytes):
         absolute file offset in bytes and number of bytes to read
     n_bytes : int
         total number of bytes that will be read
+    lock : {None, threading.Lock, lock-like} optional
+        If provided, used to ensure that paired calls to ``seek`` and ``read``
+        cannot be interrupted by another thread accessing the same ``fileobj``.
+        Each thread which accesses the same file via ``read_segments`` must
+        share a lock in order to ensure that the file access is thread-safe.
+        A lock does not need to be provided for single-threaded access. The
+        default value (``None``) results in a lock-like object  (a
+        ``_NullLock``) which does not do anything.
 
     Returns
     -------
@@ -641,22 +665,28 @@ def read_segments(fileobj, segments, n_bytes):
         object implementing buffer protocol, such as byte string or ndarray or
         mmap or ctypes ``c_char_array``
     """
+    # Make a lock-like thing to make the code below a bit nicer
+    if lock is None:
+        lock = _NullLock()
+
     if len(segments) == 0:
         if n_bytes != 0:
             raise ValueError("No segments, but non-zero n_bytes")
         return b''
     if len(segments) == 1:
         offset, length = segments[0]
-        fileobj.seek(offset)
-        bytes = fileobj.read(length)
+        with lock:
+            fileobj.seek(offset)
+            bytes = fileobj.read(length)
         if len(bytes) != n_bytes:
             raise ValueError("Whoops, not enough data in file")
         return bytes
     # More than one segment
     bytes = mmap(-1, n_bytes)
     for offset, length in segments:
-        fileobj.seek(offset)
-        bytes.write(fileobj.read(length))
+        with lock:
+            fileobj.seek(offset)
+            bytes.write(fileobj.read(length))
     if bytes.tell() != n_bytes:
         raise ValueError("Oh dear, n_bytes does not look right")
     return bytes
@@ -700,7 +730,7 @@ def _simple_fileslice(fileobj, sliceobj, shape, dtype, offset=0, order='C',
 
 
 def fileslice(fileobj, sliceobj, shape, dtype, offset=0, order='C',
-              heuristic=threshold_heuristic):
+              heuristic=threshold_heuristic, lock=None):
     """ Slice array in `fileobj` using `sliceobj` slicer and array definitions
 
     `fileobj` contains the contiguous binary data for an array ``A`` of shape,
@@ -719,22 +749,32 @@ def fileslice(fileobj, sliceobj, shape, dtype, offset=0, order='C',
     Parameters
     ----------
     fileobj : file-like object
-        binary file-like object. Implements ``read`` and ``seek``
+        file-like object, opened for reading in binary mode. Implements
+        ``read`` and ``seek``.
     sliceobj : object
-        something that can be used to slice an array as in ``arr[sliceobj]``
+        something that can be used to slice an array as in ``arr[sliceobj]``.
     shape : sequence
-        shape of full array inside `fileobj`
-    dtype : dtype object
-        dtype of array inside `fileobj`
+        shape of full array inside `fileobj`.
+    dtype : dtype specifier
+        dtype of array inside `fileobj`, or input to ``numpy.dtype`` to specify
+        array dtype.
     offset : int, optional
         offset of array data within `fileobj`
     order : {'C', 'F'}, optional
-        memory layout of array in `fileobj`
+        memory layout of array in `fileobj`.
     heuristic : callable, optional
         function taking slice object, axis length, stride length as arguments,
         returning one of 'full', 'contiguous', None.  See
         :func:`optimize_slicer` and see :func:`threshold_heuristic` for an
         example.
+    lock : {None, threading.Lock, lock-like} optional
+        If provided, used to ensure that paired calls to ``seek`` and ``read``
+        cannot be interrupted by another thread accessing the same ``fileobj``.
+        Each thread which accesses the same file via ``read_segments`` must
+        share a lock in order to ensure that the file access is thread-safe.
+        A lock does not need to be provided for single-threaded access. The
+        default value (``None``) results in a lock-like object  (a
+        ``_NullLock``) which does not do anything.
 
     Returns
     -------
@@ -743,12 +783,13 @@ def fileslice(fileobj, sliceobj, shape, dtype, offset=0, order='C',
     """
     if is_fancy(sliceobj):
         raise ValueError("Cannot handle fancy indexing")
-    itemsize = dtype.itemsize
+    dtype = np.dtype(dtype)
+    itemsize = int(dtype.itemsize)
     segments, sliced_shape, post_slicers = calc_slicedefs(
         sliceobj, shape, itemsize, offset, order)
     n_bytes = reduce(operator.mul, sliced_shape, 1) * itemsize
-    bytes = read_segments(fileobj, segments, n_bytes)
-    sliced = np.ndarray(sliced_shape, dtype, buffer=bytes, order=order)
+    arr_data = read_segments(fileobj, segments, n_bytes, lock)
+    sliced = np.ndarray(sliced_shape, dtype, buffer=arr_data, order=order)
     return sliced[post_slicers]
 
 
