@@ -9,21 +9,16 @@
 # module imports
 """ Utilities to load and save image objects """
 
+import os.path as op
 import numpy as np
 
-from .filename_parser import types_filenames, splitext_addext
-from .volumeutils import BinOpener, Opener
-from .analyze import AnalyzeImage
-from .spm2analyze import Spm2AnalyzeImage
-from .nifti1 import Nifti1Image, Nifti1Pair, header_dtype as ni1_hdr_dtype
-from .nifti2 import Nifti2Image, Nifti2Pair
-from .minc1 import Minc1Image
-from .minc2 import Minc2Image
-from .freesurfer import MGHImage
-from .fileholders import FileHolderError
-from .spatialimages import ImageFileError
-from .imageclasses import class_map, ext_map
+from .filename_parser import splitext_addext
+from .openers import ImageOpener
+from .filebasedimages import ImageFileError
+from .imageclasses import all_image_classes
 from .arrayproxy import is_proxy
+from .py3k import FileNotFoundError
+from .deprecated import deprecate_with_version
 
 
 def load(filename, **kwargs):
@@ -41,9 +36,22 @@ def load(filename, **kwargs):
     img : ``SpatialImage``
        Image of guessed type
     '''
-    return guessed_image_type(filename).from_filename(filename, **kwargs)
+    if not op.exists(filename):
+        raise FileNotFoundError("No such file: '%s'" % filename)
+    sniff = None
+    for image_klass in all_image_classes:
+        is_valid, sniff = image_klass.path_maybe_image(filename, sniff)
+        if is_valid:
+            img = image_klass.from_filename(filename, **kwargs)
+            return img
+
+    raise ImageFileError('Cannot work out file type of "%s"' %
+                         filename)
 
 
+@deprecate_with_version('guessed_image_type deprecated.'
+                        '2.1',
+                        '4.0')
 def guessed_image_type(filename):
     """ Guess image type from file `filename`
 
@@ -57,39 +65,14 @@ def guessed_image_type(filename):
     image_class : class
         Class corresponding to guessed image type
     """
-    froot, ext, trailing = splitext_addext(filename, ('.gz', '.bz2'))
-    lext = ext.lower()
-    try:
-        img_type = ext_map[lext]
-    except KeyError:
-        raise ImageFileError('Cannot work out file type of "%s"' %
-                             filename)
-    if lext in ('.mgh', '.mgz', '.par'):
-        klass = class_map[img_type]['class']
-    elif lext == '.mnc':
-        # Look for HDF5 signature for MINC2
-        # https://www.hdfgroup.org/HDF5/doc/H5.format.html
-        with Opener(filename) as fobj:
-            signature = fobj.read(4)
-            klass = Minc2Image if signature == b'\211HDF' else Minc1Image
-    elif lext == '.nii':
-        with BinOpener(filename) as fobj:
-            binaryblock = fobj.read(348)
-        ft = which_analyze_type(binaryblock)
-        klass = Nifti2Image if ft == 'nifti2' else Nifti1Image
-    else: # might be nifti 1 or 2 pair or analyze of some sort
-        files_types = (('image','.img'), ('header','.hdr'))
-        filenames = types_filenames(filename, files_types)
-        with BinOpener(filenames['header']) as fobj:
-            binaryblock = fobj.read(348)
-        ft = which_analyze_type(binaryblock)
-        if ft == 'nifti2':
-            klass = Nifti2Pair
-        elif ft == 'nifti1':
-            klass = Nifti1Pair
-        else:
-            klass = Spm2AnalyzeImage
-    return klass
+    sniff = None
+    for image_klass in all_image_classes:
+        is_valid, sniff = image_klass.path_maybe_image(filename, sniff)
+        if is_valid:
+            return image_klass
+
+    raise ImageFileError('Cannot work out file type of "%s"' %
+                         filename)
 
 
 def save(img, filename):
@@ -106,40 +89,73 @@ def save(img, filename):
     -------
     None
     '''
+
+    # Save the type as expected
     try:
         img.to_filename(filename)
     except ImageFileError:
         pass
     else:
         return
+
+    # Be nice to users by making common implicit conversions
     froot, ext, trailing = splitext_addext(filename, ('.gz', '.bz2'))
+    lext = ext.lower()
+
     # Special-case Nifti singles and Pairs
-    if type(img) == Nifti1Image and ext in ('.img', '.hdr'):
+    # Inline imports, as this module really shouldn't reference any image type
+    from .nifti1 import Nifti1Image, Nifti1Pair
+    from .nifti2 import Nifti2Image, Nifti2Pair
+
+    klass = None
+    converted = None
+
+    if type(img) == Nifti1Image and lext in ('.img', '.hdr'):
         klass = Nifti1Pair
-    elif type(img) == Nifti2Image and ext in ('.img', '.hdr'):
+    elif type(img) == Nifti2Image and lext in ('.img', '.hdr'):
         klass = Nifti2Pair
-    elif type(img) == Nifti1Pair and ext == '.nii':
+    elif type(img) == Nifti1Pair and lext == '.nii':
         klass = Nifti1Image
-    elif type(img) == Nifti2Pair and ext == '.nii':
+    elif type(img) == Nifti2Pair and lext == '.nii':
         klass = Nifti2Image
-    else:
-        img_type = ext_map[ext]
-        klass = class_map[img_type]['class']
-    converted = klass.from_image(img)
+    else:  # arbitrary conversion
+        valid_klasses = [klass for klass in all_image_classes
+                         if ext in klass.valid_exts]
+        if not valid_klasses:  # if list is empty
+            raise ImageFileError('Cannot work out file type of "%s"' %
+                                 filename)
+
+        # Got a list of valid extensions, but that's no guarantee
+        #   the file conversion will work. So, try each image
+        #   in order...
+        for klass in valid_klasses:
+            try:
+                converted = klass.from_image(img)
+                break
+            except Exception as e:
+                continue
+        # ... and if none of them work, raise an error.
+        if converted is None:
+            raise e
+
+    # Here, we either have a klass or a converted image.
+    if converted is None:
+        converted = klass.from_image(img)
     converted.to_filename(filename)
 
 
-@np.deprecate_with_doc('Please use ``img.dataobj.get_unscaled()`` '
-                       'instead')
+@deprecate_with_version('read_img_data deprecated.'
+                        'Please use ``img.dataobj.get_unscaled()`` instead.'
+                        '2.0.1',
+                        '4.0')
 def read_img_data(img, prefer='scaled'):
     """ Read data from image associated with files
 
-    We've deprecated this function and will remove it soon. If you want
-    unscaled data, please use ``img.dataobj.get_unscaled()`` instead.  If you
-    want scaled data, use ``img.get_data()`` (which will cache the loaded
-    array) or ``np.array(img.dataobj)`` (which won't cache the array). If you
-    want to load the data as for a modified header, save the image with the
-    modified header, and reload.
+    If you want unscaled data, please use ``img.dataobj.get_unscaled()``
+    instead.  If you want scaled data, use ``img.get_data()`` (which will cache
+    the loaded array) or ``np.array(img.dataobj)`` (which won't cache the
+    array). If you want to load the data as for a modified header, save the
+    image with the modified header, and reload.
 
     Parameters
     ----------
@@ -209,20 +225,23 @@ def read_img_data(img, prefer='scaled'):
             hdr.set_data_offset(dao.offset)
         if default_scaling and (dao.slope, dao.inter) != (1, 0):
             hdr.set_slope_inter(dao.slope, dao.inter)
-    with BinOpener(img_file_like) as fileobj:
+    with ImageOpener(img_file_like) as fileobj:
         if prefer == 'scaled':
             return hdr.data_from_fileobj(fileobj)
         return hdr.raw_data_from_fileobj(fileobj)
 
 
+@deprecate_with_version('which_analyze_type deprecated.'
+                        '2.1',
+                        '4.0')
 def which_analyze_type(binaryblock):
     """ Is `binaryblock` from NIfTI1, NIfTI2 or Analyze header?
 
     Parameters
     ----------
     binaryblock : bytes
-        The `binaryblock` is 348 bytes that might be NIfTI1, NIfTI2, Analyze, or
-        None of the the above.
+        The `binaryblock` is 348 bytes that might be NIfTI1, NIfTI2, Analyze,
+        or None of the the above.
 
     Returns
     -------
@@ -242,13 +261,14 @@ def which_analyze_type(binaryblock):
     * if ``sizeof_hdr`` is 348 or byteswapped 348 assume Analyze
     * Return None
     """
-    hdr = np.ndarray(shape=(), dtype=ni1_hdr_dtype, buffer=binaryblock)
-    bs_hdr = hdr.byteswap()
-    sizeof_hdr = hdr['sizeof_hdr']
-    bs_sizeof_hdr = bs_hdr['sizeof_hdr']
+    from .nifti1 import header_dtype
+    hdr_struct = np.ndarray(shape=(), dtype=header_dtype, buffer=binaryblock)
+    bs_hdr_struct = hdr_struct.byteswap()
+    sizeof_hdr = hdr_struct['sizeof_hdr']
+    bs_sizeof_hdr = bs_hdr_struct['sizeof_hdr']
     if 540 in (sizeof_hdr, bs_sizeof_hdr):
         return 'nifti2'
-    if hdr['magic'] in (b'ni1', b'n+1'):
+    if hdr_struct['magic'] in (b'ni1', b'n+1'):
         return 'nifti1'
     if 348 in (sizeof_hdr, bs_sizeof_hdr):
         return 'analyze'
