@@ -95,8 +95,8 @@ class Minc1File(object):
     def get_zooms(self):
         """ Get real-world sizes of voxels """
         # zooms must be positive; but steps in MINC can be negative
-        return tuple(
-            [abs(float(dim.step)) for dim in self._dims])
+        return tuple([abs(float(dim.step)) if hasattr(dim, 'step') else 1.0
+                      for dim in self._dims])
 
     def get_affine(self):
         nspatial = len(self._spatial_dims)
@@ -106,13 +106,11 @@ class Minc1File(object):
         dim_names = list(self._dim_names)  # for indexing in loop
         for i, name in enumerate(self._spatial_dims):
             dim = self._dims[dim_names.index(name)]
-            try:
-                dir_cos = dim.direction_cosines
-            except AttributeError:
-                dir_cos = _default_dir_cos[name]
-            rot_mat[:, i] = dir_cos
-            steps[i] = dim.step
-            starts[i] = dim.start
+            rot_mat[:, i] = (dim.direction_cosines
+                             if hasattr(dim, 'direction_cosines')
+                             else _default_dir_cos[name])
+            steps[i] = dim.step if hasattr(dim, 'step') else 1.0
+            starts[i] = dim.start if hasattr(dim, 'start') else 0.0
         origin = np.dot(rot_mat, starts)
         aff = np.eye(nspatial + 1)
         aff[:nspatial, :nspatial] = rot_mat * steps
