@@ -11,6 +11,7 @@ than an AttributeError - breaking the 'properties manifesto'.   So, any
 processing that needs to raise an error, should be in a method, rather
 than in a property, or property-like thing.
 """
+from __future__ import division
 
 import operator
 
@@ -18,8 +19,9 @@ import numpy as np
 
 from . import csareader as csar
 from .dwiparams import B2q, nearest_pos_semi_def, q2bg
-from ..volumeutils import BinOpener
+from ..openers import ImageOpener
 from ..onetime import setattr_on_read as one_time
+from ..pydicom_compat import pydicom
 
 
 class WrapperError(Exception):
@@ -49,12 +51,9 @@ def wrapper_from_file(file_like, *args, **kwargs):
     dcm_w : ``dicomwrappers.Wrapper`` or subclass
        DICOM wrapper corresponding to DICOM data type
     """
-    try:
-        from dicom import read_file
-    except ImportError:
-        from pydicom.dicomio import read_file
+    from ..pydicom_compat import read_file
 
-    with BinOpener(file_like) as fobj:
+    with ImageOpener(file_like) as fobj:
         dcm_data = read_file(fobj, *args, **kwargs)
     return wrapper_from_data(dcm_data)
 
@@ -185,7 +184,8 @@ class Wrapper(object):
         # motivated in ``doc/source/notebooks/ata_error.ipynb``, and from
         # discussion at https://github.com/nipy/nibabel/pull/156
         if not np.allclose(np.eye(3), np.dot(R, R.T), atol=5e-5):
-            raise WrapperPrecisionError('Rotation matrix not nearly orthogonal')
+            raise WrapperPrecisionError('Rotation matrix not nearly '
+                                        'orthogonal')
         return R
 
     @one_time
@@ -277,7 +277,7 @@ class Wrapper(object):
 
     def __getitem__(self, key):
         """ Return values from DICOM object"""
-        if not key in self.dcm_data:
+        if key not in self.dcm_data:
             raise KeyError('"%s" not in self.dcm_data' % key)
         return self.dcm_data.get(key)
 
@@ -374,7 +374,8 @@ class Wrapper(object):
         return True
 
     def _scale_data(self, data):
-        # depending on pydicom and dicom files, values might need casting from Decimal to float
+        # depending on pydicom and dicom files, values might need casting from
+        # Decimal to float
         scale = float(self.get('RescaleSlope', 1))
         offset = float(self.get('RescaleIntercept', 0))
         return self._apply_scale_offset(data, scale, offset)
@@ -412,7 +413,18 @@ class Wrapper(object):
 class MultiframeWrapper(Wrapper):
     """Wrapper for Enhanced MR Storage SOP Class
 
-    tested with Philips' Enhanced DICOM implementation
+    Tested with Philips' Enhanced DICOM implementation.
+
+    The specification for the Enhanced MR image IOP / SOP began life as `DICOM
+    supplement 49 <ftp://medical.nema.org/medical/dicom/final/sup49_ft.pdf>`_,
+    but as of 2016 it is part of the standard. In particular see:
+
+    * `A.36 Enhanced MR Information Object Definitions
+      <http://dicom.nema.org/medical/dicom/current/output/pdf/part03.pdf#sect_A.36>`_;
+    * `C.7.6.16 Multi-Frame Functional Groups Module
+      <http://dicom.nema.org/medical/dicom/current/output/pdf/part03.pdf#sect_C.7.6.16>`_;
+    * `C.7.6.17 Multi-Frame Dimension Module
+      <http://dicom.nema.org/medical/dicom/current/output/pdf/part03.pdf#sect_C.7.6.17>`_.
 
     Attributes
     ----------
@@ -461,7 +473,32 @@ class MultiframeWrapper(Wrapper):
 
     @one_time
     def image_shape(self):
-        """The array shape as it will be returned by ``get_data()``"""
+        """The array shape as it will be returned by ``get_data()``
+
+        The shape is determined by the *Rows* DICOM attribute, *Columns*
+        DICOM attribute, and the set of frame indices given by the
+        *FrameContentSequence[0].DimensionIndexValues* DICOM attribute of each
+        element in the *PerFrameFunctionalGroupsSequence*.  The first two
+        axes of the returned shape correspond to the rows, and columns
+        respectively. The remaining axes correspond to those of the frame
+        indices with order preserved.
+
+        What each axis in the frame indices refers to is given by the
+        corresponding entry in the *DimensionIndexSequence* DICOM attribute.
+        **WARNING**: Any axis refering to the *StackID* DICOM attribute will
+        have been removed from the frame indices in determining the shape. This
+        is because only a file containing a single stack is currently allowed by
+        this wrapper.
+
+        References
+        ----------
+        * C.7.6.16 Multi-Frame Functional Groups Module:
+          http://dicom.nema.org/medical/dicom/current/output/pdf/part03.pdf#sect_C.7.6.16
+        * C.7.6.17 Multi-Frame Dimension Module:
+          http://dicom.nema.org/medical/dicom/current/output/pdf/part03.pdf#sect_C.7.6.17
+        * Diagram of DimensionIndexSequence and DimensionIndexValues:
+          http://dicom.nema.org/medical/dicom/current/output/pdf/part03.pdf#figure_C.7.6.17-1
+        """
         rows, cols = self.get('Rows'), self.get('Columns')
         if None in (rows, cols):
             raise WrapperError("Rows and/or Columns are empty.")
@@ -471,12 +508,25 @@ class MultiframeWrapper(Wrapper):
         frame_indices = np.array(
             [frame.FrameContentSequence[0].DimensionIndexValues
              for frame in self.frames])
-        n_dim = frame_indices.shape[1] + 1
-        # Check there is only one multiframe stack index
-        if np.any(np.diff(frame_indices[:, 0])):
-            raise WrapperError("File contains more than one StackID. Cannot handle multi-stack files")
+        # Check that there is only one multiframe stack index
+        stack_ids = set(frame.FrameContentSequence[0].StackID
+                        for frame in self.frames)
+        if len(stack_ids) > 1:
+            raise WrapperError("File contains more than one StackID. "
+                               "Cannot handle multi-stack files")
+        # Determine if one of the dimension indices refers to the stack id
+        dim_seq = [dim.DimensionIndexPointer
+                   for dim in self.get('DimensionIndexSequence')]
+        stackid_tag = pydicom.datadict.tag_for_name('StackID')
+        # remove the stack id axis if present
+        if stackid_tag in dim_seq:
+            stackid_dim_idx = dim_seq.index(stackid_tag)
+            frame_indices = np.delete(frame_indices, stackid_dim_idx, axis=1)
+        # account for the 2 additional dimensions (row and column) not included
+        # in the indices
+        n_dim = frame_indices.shape[1] + 2
         # Store frame indices
-        self._frame_indices = frame_indices[:, 1:]
+        self._frame_indices = frame_indices
         if n_dim < 4:  # 3D volume
             return rows, cols, n_frames
         # More than 3 dimensions
@@ -484,7 +534,8 @@ class MultiframeWrapper(Wrapper):
         shape = (rows, cols) + tuple(ns_unique)
         n_vols = np.prod(shape[3:])
         if n_frames != n_vols * shape[2]:
-            raise WrapperError("Calculated shape does not match number of frames.")
+            raise WrapperError("Calculated shape does not match number of "
+                               "frames.")
         return tuple(shape)
 
     @one_time
@@ -498,7 +549,8 @@ class MultiframeWrapper(Wrapper):
             try:
                 iop = self.frames[0].PlaneOrientationSequence[0].ImageOrientationPatient
             except AttributeError:
-                raise WrapperError("Not enough information for image_orient_patient")
+                raise WrapperError("Not enough information for "
+                                   "image_orient_patient")
         if iop is None:
             return None
         iop = np.array(list(map(float, iop)))
@@ -614,8 +666,8 @@ class SiemensWrapper(Wrapper):
 
     @one_time
     def slice_normal(self):
-        #The std_slice_normal comes from the cross product of the directions
-        #in the ImageOrientationPatient
+        # The std_slice_normal comes from the cross product of the directions
+        # in the ImageOrientationPatient
         std_slice_normal = super(SiemensWrapper, self).slice_normal
         csa_slice_normal = csar.get_slice_normal(self.csa_header)
         if std_slice_normal is None and csa_slice_normal is None:
@@ -625,12 +677,12 @@ class SiemensWrapper(Wrapper):
         elif csa_slice_normal is None:
             return std_slice_normal
         else:
-            #Make sure the two normals are very close to parallel unit vectors
+            # Make sure the two normals are very close to parallel unit vectors
             dot_prod = np.dot(csa_slice_normal, std_slice_normal)
             assert np.allclose(np.fabs(dot_prod), 1.0, atol=1e-5)
-            #Use the slice normal computed with the cross product as it will
-            #always be the most orthogonal, but take the sign from the CSA
-            #slice normal
+            # Use the slice normal computed with the cross product as it will
+            # always be the most orthogonal, but take the sign from the CSA
+            # slice normal
             if dot_prod < 0:
                 return -std_slice_normal
             else:
@@ -641,7 +693,7 @@ class SiemensWrapper(Wrapper):
         """ Add ICE dims from CSA header to signature """
         signature = super(SiemensWrapper, self).series_signature
         ice = csar.get_ice_dims(self.csa_header)
-        if not ice is None:
+        if ice is not None:
             ice = ice[:6] + ice[8:9]
         signature['ICE_Dims'] = (ice, lambda x, y: x == y)
         return signature
@@ -719,7 +771,7 @@ class MosaicWrapper(SiemensWrapper):
     Adds attributes:
 
     * n_mosaic : int
-    * mosaic_size : float
+    * mosaic_size : int
     """
     is_mosaic = True
 
@@ -753,7 +805,7 @@ class MosaicWrapper(SiemensWrapper):
                                    'header; is this really '
                                    'Siemens mosiac data?')
         self.n_mosaic = n_mosaic
-        self.mosaic_size = np.ceil(np.sqrt(n_mosaic))
+        self.mosaic_size = int(np.ceil(np.sqrt(n_mosaic)))
 
     @one_time
     def image_shape(self):
@@ -820,19 +872,20 @@ class MosaicWrapper(SiemensWrapper):
         -----
         The apparent image in the DICOM file is a 2D array that consists of
         blocks, that are the output 2D slices.  Let's call the original array
-        the *slab*, and the contained slices *slices*.   The slices are of pixel
-        dimension ``n_slice_rows`` x ``n_slice_cols``.  The slab is of pixel
-        dimension ``n_slab_rows`` x ``n_slab_cols``.  Because the arrangement of
-        blocks in the slab is defined as being square, the number of blocks per
-        slab row and slab column is the same.  Let ``n_blocks`` be the number of
-        blocks contained in the slab.  There is also ``n_slices`` - the number
-        of slices actually collected, some number <= ``n_blocks``.  We have the
-        value ``n_slices`` from the 'NumberOfImagesInMosaic' field of the
-        Siemens private (CSA) header.  ``n_row_blocks`` and ``n_col_blocks`` are
-        therefore given by ``ceil(sqrt(n_slices))``, and ``n_blocks`` is
-        ``n_row_blocks ** 2``.  Also ``n_slice_rows == n_slab_rows /
-        n_row_blocks``, etc.  Using these numbers we can therefore reconstruct
-        the slices from the 2D DICOM pixel array.
+        the *slab*, and the contained slices *slices*.   The slices are of
+        pixel dimension ``n_slice_rows`` x ``n_slice_cols``.  The slab is of
+        pixel dimension ``n_slab_rows`` x ``n_slab_cols``.  Because the
+        arrangement of blocks in the slab is defined as being square, the
+        number of blocks per slab row and slab column is the same.  Let
+        ``n_blocks`` be the number of blocks contained in the slab.  There is
+        also ``n_slices`` - the number of slices actually collected, some
+        number <= ``n_blocks``.  We have the value ``n_slices`` from the
+        'NumberOfImagesInMosaic' field of the Siemens private (CSA) header.
+        ``n_row_blocks`` and ``n_col_blocks`` are therefore given by
+        ``ceil(sqrt(n_slices))``, and ``n_blocks`` is ``n_row_blocks ** 2``.
+        Also ``n_slice_rows == n_slab_rows / n_row_blocks``, etc.  Using these
+        numbers we can therefore reconstruct the slices from the 2D DICOM pixel
+        array.
         """
         shape = self.image_shape
         if shape is None:
