@@ -1,9 +1,9 @@
 """
 Python 3 compatibility tools.
 
-Copied from numpy/compat/py3k
+Copied from numpy/compat/py3k.
 
-Please prefer the routines in externals/six.py when possible
+Please prefer the routines in the six module when possible.
 
 BSD license
 """
