@@ -26,36 +26,11 @@ from .header import Field
 MAX_NB_NAMED_SCALARS_PER_POINT = 10
 MAX_NB_NAMED_PROPERTIES_PER_STREAMLINE = 10
 
-# See http://docs.scipy.org/doc/numpy/reference/arrays.dtypes.html
-header_1_dtd = [(Field.MAGIC_NUMBER, 'S6'),
-                (Field.DIMENSIONS, 'h', 3),
-                (Field.VOXEL_SIZES, 'f4', 3),
-                (Field.ORIGIN, 'f4', 3),
-                (Field.NB_SCALARS_PER_POINT, 'h'),
-                ('scalar_name', 'S20', MAX_NB_NAMED_SCALARS_PER_POINT),
-                (Field.NB_PROPERTIES_PER_STREAMLINE, 'h'),
-                ('property_name', 'S20',
-                 MAX_NB_NAMED_PROPERTIES_PER_STREAMLINE),
-                ('reserved', 'S508'),
-                (Field.VOXEL_ORDER, 'S4'),
-                ('pad2', 'S4'),
-                ('image_orientation_patient', 'f4', 6),
-                ('pad1', 'S2'),
-                ('invert_x', 'S1'),
-                ('invert_y', 'S1'),
-                ('invert_z', 'S1'),
-                ('swap_xy', 'S1'),
-                ('swap_yz', 'S1'),
-                ('swap_zx', 'S1'),
-                (Field.NB_STREAMLINES, 'i4'),
-                ('version', 'i4'),
-                ('hdr_size', 'i4'),
-                ]
-
-# Version 2 adds a 4x4 matrix giving the affine transformtation going
+# Version 2 adds a 4x4 matrix giving the affine transformation going
 # from voxel coordinates in the referenced 3D voxel matrix, to xyz
 # coordinates (axes L->R, P->A, I->S). If (0 based) value [3, 3] from
 # this matrix is 0, this means the matrix is not recorded.
+# See http://docs.scipy.org/doc/numpy/reference/arrays.dtypes.html
 header_2_dtd = [(Field.MAGIC_NUMBER, 'S6'),
                 (Field.DIMENSIONS, 'h', 3),
                 (Field.VOXEL_SIZES, 'f4', 3),
@@ -83,7 +58,6 @@ header_2_dtd = [(Field.MAGIC_NUMBER, 'S6'),
                 ]
 
 # Full header numpy dtypes
-header_1_dtype = np.dtype(header_1_dtd)
 header_2_dtype = np.dtype(header_2_dtd)
 
 
@@ -98,8 +72,8 @@ def get_affine_trackvis_to_rasmm(header):
 
     Parameters
     ----------
-    header : dict
-        Dict containing trackvis header.
+    header : dict or ndarray
+        Dict or numpy structured array containing trackvis header.
 
     Returns
     -------
@@ -127,9 +101,12 @@ def get_affine_trackvis_to_rasmm(header):
     # If the voxel order implied by the affine does not match the voxel
     # order in the TRK header, change the orientation.
     # voxel (header) -> voxel (affine)
-    header_ornt = asstr(header[Field.VOXEL_ORDER])
+    vox_order = header[Field.VOXEL_ORDER]
+    # Input header can be dict or structured array
+    if hasattr(vox_order, 'item'):  # structured array
+        vox_order = header[Field.VOXEL_ORDER].item()
     affine_ornt = "".join(aff2axcodes(header[Field.VOXEL_TO_RASMM]))
-    header_ornt = axcodes2ornt(header_ornt)
+    header_ornt = axcodes2ornt(vox_order.decode('latin1'))
     affine_ornt = axcodes2ornt(affine_ornt)
     ornt = nib.orientations.ornt_transform(header_ornt, affine_ornt)
     M = nib.orientations.inv_ornt_aff(ornt, header[Field.DIMENSIONS])
@@ -225,28 +202,11 @@ def decode_value_from_name(encoded_name):
     return name, value
 
 
-def create_empty_header():
-    """ Return an empty compliant TRK header. """
-    header = np.zeros(1, dtype=header_2_dtype)
-
-    # Default values
-    header[Field.MAGIC_NUMBER] = TrkFile.MAGIC_NUMBER
-    header[Field.VOXEL_SIZES] = np.array((1, 1, 1), dtype="f4")
-    header[Field.DIMENSIONS] = np.array((1, 1, 1), dtype="h")
-    header[Field.VOXEL_TO_RASMM] = np.eye(4, dtype="f4")
-    header[Field.VOXEL_ORDER] = b"RAS"
-    header['version'] = 2
-    header['hdr_size'] = TrkFile.HEADER_SIZE
-
-    return header
-
-
 class TrkFile(TractogramFile):
     """ Convenience class to encapsulate TRK file format.
 
     Notes
     -----
-
     TrackVis (so its file format: TRK) considers the streamline coordinate
     (0,0,0) to be in the corner of the voxel whereas NiBabel's streamlines
     internal representation (Voxel space) assumes (0,0,0) to be in the
@@ -278,10 +238,6 @@ class TrkFile(TractogramFile):
         and *mm* space where coordinate (0,0,0) refers to the center
         of the voxel.
         """
-        if header is None:
-            header_rec = create_empty_header()
-            header = dict(zip(header_rec.dtype.names, header_rec[0]))
-
         super(TrkFile, self).__init__(tractogram, header)
 
     @classmethod
@@ -308,6 +264,30 @@ class TrkFile(TractogramFile):
             f.seek(-magic_len, os.SEEK_CUR)
             return magic_number == cls.MAGIC_NUMBER
 
+    @classmethod
+    def _default_structarr(cls):
+        """ Return an empty compliant TRK header as numpy structured array
+        """
+        st_arr = np.zeros((), dtype=header_2_dtype)
+
+        # Default values
+        st_arr[Field.MAGIC_NUMBER] = cls.MAGIC_NUMBER
+        st_arr[Field.VOXEL_SIZES] = np.array((1, 1, 1), dtype="f4")
+        st_arr[Field.DIMENSIONS] = np.array((1, 1, 1), dtype="h")
+        st_arr[Field.VOXEL_TO_RASMM] = np.eye(4, dtype="f4")
+        st_arr[Field.VOXEL_ORDER] = b"RAS"
+        st_arr['version'] = 2
+        st_arr['hdr_size'] = cls.HEADER_SIZE
+
+        return st_arr
+
+    @classmethod
+    def create_empty_header(cls):
+        """ Return an empty compliant TRK header as dict
+        """
+        st_arr = cls._default_structarr()
+        return dict(zip(st_arr.dtype.names, st_arr.tolist()))
+
     @classmethod
     def load(cls, fileobj, lazy_load=False):
         """ Loads streamlines from a filename or file-like object.
@@ -341,8 +321,8 @@ class TrkFile(TractogramFile):
         data_per_point_slice = {}
         if hdr[Field.NB_SCALARS_PER_POINT] > 0:
             cpt = 0
-            for scalar_name in hdr['scalar_name']:
-                scalar_name, nb_scalars = decode_value_from_name(scalar_name)
+            for scalar_field in hdr['scalar_name']:
+                scalar_name, nb_scalars = decode_value_from_name(scalar_field)
 
                 if nb_scalars == 0:
                     continue
@@ -358,8 +338,8 @@ class TrkFile(TractogramFile):
         data_per_streamline_slice = {}
         if hdr[Field.NB_PROPERTIES_PER_STREAMLINE] > 0:
             cpt = 0
-            for property_name in hdr['property_name']:
-                results = decode_value_from_name(property_name)
+            for property_field in hdr['property_name']:
+                results = decode_value_from_name(property_field)
                 property_name, nb_properties = results
 
                 if nb_properties == 0:
@@ -415,7 +395,7 @@ class TrkFile(TractogramFile):
             of the TRK header data).
         """
         # Enforce little-endian byte order for header
-        header = create_empty_header().newbyteorder('<')
+        header = self._default_structarr().newbyteorder('<')
 
         # Override hdr's fields by those contained in `header`.
         for k, v in self.header.items():
@@ -433,7 +413,6 @@ class TrkFile(TractogramFile):
         nb_scalars = 0
         nb_properties = 0
 
-        header = header[0]
         with Opener(fileobj, mode="wb") as f:
             # Keep track of the beginning of the header.
             beginning = f.tell()
@@ -584,8 +563,8 @@ class TrkFile(TractogramFile):
                                                  TrkFile.HEADER_SIZE))
 
             if header_rec['version'] == 1:
-                header_rec = np.fromstring(string=header_str,
-                                           dtype=header_1_dtype)
+                # There is no 4x4 matrix for voxel to RAS transformation.
+                header_rec[Field.VOXEL_TO_RASMM] = np.zeros((4, 4))
             elif header_rec['version'] == 2:
                 pass  # Nothing more to do.
             else:
@@ -623,7 +602,7 @@ class TrkFile(TractogramFile):
             # Keep the file position where the data begin.
             header['_offset_data'] = f.tell()
 
-        # Set the file position where it was, if it was previously open
+        # Set the file position where it was, if it was previously open.
         if start_position is not None:
             fileobj.seek(start_position, os.SEEK_SET)
 
@@ -724,12 +703,17 @@ class TrkFile(TractogramFile):
                 hdr_field = getattr(Field, attr)
                 if hdr_field in vars:
                     vars[attr] = vars[hdr_field]
-        vars['scalar_names'] = '\n  '.join([asstr(s)
-                                            for s in vars['scalar_name']
-                                            if len(s) > 0])
-        vars['property_names'] = "\n  ".join([asstr(s)
-                                              for s in vars['property_name']
-                                              if len(s) > 0])
+
+        nb_scalars = self.header[Field.NB_SCALARS_PER_POINT]
+        scalar_names = [asstr(s)
+                        for s in vars['scalar_name'][:nb_scalars]
+                        if len(s) > 0]
+        vars['scalar_names'] = '\n  '.join(scalar_names)
+        nb_properties = self.header[Field.NB_PROPERTIES_PER_STREAMLINE]
+        property_names = [asstr(s)
+                          for s in vars['property_name'][:nb_properties]
+                          if len(s) > 0]
+        vars['property_names'] = "\n  ".join(property_names)
         # Make all byte strings into strings
         # Fixes recursion error on Python 3.3
         vars = dict((k, asstr(v) if hasattr(v, 'decode') else v)
@@ -739,11 +723,11 @@ MAGIC NUMBER: {MAGIC_NUMBER}
 v.{version}
 dim: {DIMENSIONS}
 voxel_sizes: {VOXEL_SIZES}
-orgin: {ORIGIN}
+origin: {ORIGIN}
 nb_scalars: {NB_SCALARS_PER_POINT}
-scalar_name:\n  {scalar_names}
+scalar_names:\n  {scalar_names}
 nb_properties: {NB_PROPERTIES_PER_STREAMLINE}
-property_name:\n  {property_names}
+property_names:\n  {property_names}
 vox_to_world:\n{VOXEL_TO_RASMM}
 voxel_order: {VOXEL_ORDER}
 image_orientation_patient: {image_orientation_patient}
