@@ -10,12 +10,13 @@
 
 import sys
 
-import numpy as np
 
-from nose.tools import assert_raises, assert_true, assert_equal
+from nose.tools import assert_equal
+from nose.tools import assert_true
 
 from ..volumeutils import (endian_codes, native_code, swapped_code)
 
+
 def test_native_swapped():
     native_is_le = sys.byteorder == 'little'
     if native_is_le:
