@@ -11,16 +11,18 @@ from .. import (Spm99AnalyzeImage, Spm2AnalyzeImage,
                 Nifti1Pair, Nifti1Image,
                 Nifti2Pair, Nifti2Image)
 from ..loadsave import load, read_img_data
-from ..spatialimages import ImageFileError
+from ..filebasedimages import ImageFileError
 from ..tmpdirs import InTemporaryDirectory, TemporaryDirectory
 
-from .test_spm99analyze import have_scipy
+from ..optpkg import optional_package
+_, have_scipy, _ = optional_package('scipy')
 
 from numpy.testing import (assert_almost_equal,
                            assert_array_equal)
 
 from nose.tools import (assert_true, assert_false, assert_raises,
                         assert_equal, assert_not_equal)
+from ..py3k import FileNotFoundError
 
 data_path = pjoin(dirname(__file__), 'data')
 
@@ -32,7 +34,7 @@ def test_read_img_data():
                   'minc1_4d.mnc',
                   'test.mgz',
                   'tiny.mnc'
-                 ):
+                  ):
         fpath = pjoin(data_path, fname)
         img = load(fpath)
         data = img.get_data()
@@ -52,6 +54,10 @@ def test_read_img_data():
             del img
 
 
+def test_file_not_found():
+    assert_raises(FileNotFoundError, load, 'does_not_exist.nii.gz')
+
+
 def test_read_img_data_nifti():
     shape = (2, 3, 4)
     data = np.random.normal(size=shape)
