@@ -24,7 +24,7 @@ def is_tripwire(obj):
         obj.any_attribute
     except TripWireError:
         return True
-    except:
+    except Exception:
         pass
     return False
 
@@ -42,6 +42,7 @@ class TripWire(object):
         ...
     TripWireError: We do not have a_module
     """
+
     def __init__(self, msg):
         self._msg = msg
 
